@@ -22,10 +22,8 @@ from ._gaussian_mixture_model import gauss_model_2save
 class BayesianGaussianMixture(GaussianMixtureBase):
     """The class for implementing what we colloquially call the 'BGM' algorithms.
 
-    Functionality includes:
-        Cartesian or Polar Coordinates
-        Spherical, Tied, Diagonal, and Full covariance matrices
-        General or strict fits
+    Functionality includes: Cartesian or Polar Coordinates,
+    Spherical, Tied, Diagonal, or Full covariance matrices, and general or strict fits
 
     version : 0.1
 
@@ -406,6 +404,10 @@ class BayesianGaussianMixture(GaussianMixtureBase):
             information that is used by the algorithms to do the
             fits.
         """
+        if not self.clustered_:
+            raise NotImplementedError("Must run a method to cluster the "
+                                      "data before recalculating centers.")
+
         if not data_frame_object.phase_shifted_:
             shift_phase_dimension(data_frame_object)
 
@@ -692,7 +694,9 @@ class BayesianGaussianMixture(GaussianMixtureBase):
 
         Given a data frame object that has already been used
         to generate histograms for each dimension of data, this
-        method will graph a GMM fit over each dimension.
+        method will graph a GMM fit over each dimension. The
+        returned matplotlib.plyplot figure can be shown and saved
+        separately.
 
         Parameters
         ----------
@@ -752,6 +756,9 @@ class BayesianGaussianMixture(GaussianMixtureBase):
     def plot_pdf_surface(self, data_frame_object):
         """Plot the pdf of the Bayesian Gaussian mixture on a surface.
 
+        The returned matplotlib.plyplot figure can be shown and saved
+        separately.
+
         Parameters
         ----------
         data_frame_object : object from the class DataFrame
@@ -769,8 +776,8 @@ class BayesianGaussianMixture(GaussianMixtureBase):
             which is done separately.
         """
         if not self.clustered_:
-            print("Must cluster data before visualizing it.")
-            exit()
+            raise NotImplementedError("Must run a method to cluster the "
+                                      "data before visualization.")
 
         fig = plt.figure()
         ax = fig.gca(projection='3d')
@@ -864,6 +871,9 @@ class BayesianGaussianMixture(GaussianMixtureBase):
     def show_results(self, data_frame_object):
         """Display the clustering results.
 
+        The returned matplotlib.plyplot figure can be shown and saved
+        separately.
+
         This method was written by Dwaipayan Ray and Adrian Valverde.
 
         Parameters
@@ -882,6 +892,10 @@ class BayesianGaussianMixture(GaussianMixtureBase):
             The recommended file name to use when saving the plot,
             which is done separately.
         """
+        if not self.clustered_:
+            raise NotImplementedError("Must run a method to cluster the "
+                                      "data before visualization.")
+
         data_array = data_frame_object.data_array_
         n_samples = len(data_array[:, 0])
         center_array = self.centers_array_
