@@ -2,6 +2,7 @@
 
 # Author: Colin Weber
 # Contact: colin.weber.27@gmail.com
+# Contributors: Adrian Valverde and Dwaipayan Ray
 # License: MIT
 # sklearn License: BSD 3 clause
 
@@ -20,11 +21,13 @@ from joblib import Parallel, delayed
 from tqdm import tqdm
 
 
-def wt_avg_unc_number(abs_list, unc):
+def wt_avg_unc_number(abs_list: list, unc):
     """Returns a weighted average of the elements in 'abs_list'.
 
     Weights are given by the parameter 'unc'.
 
+    This method was written by Dwaipayan Ray and Adrian Valverde.
+
     Parameters
     ----------
     abs_list : list, len(abs_list) = n_samples
@@ -62,6 +65,8 @@ def gauss_model_2save(x_min, x_max, data, num_bin, *args):
 
     Also extract the Gaussian statistics for debugging convenience.
 
+    This method was written by Dwaipayan Ray and Adrian Valverde.
+
     Parameters
     ----------
     x_min : float
@@ -220,7 +225,6 @@ def gauss_model_2save(x_min, x_max, data, num_bin, *args):
         plt.plot(x_fit, y_fit, 'k-')
     plt.title('center : %.3f' % center_abs)
     plt.xlim(x_min, x_max)
-    # plt.close()
 
     return center_abs, center_err, chi_sq, red_chi_sq, sigma_abs, \
         sigma_err, height_abs, height_err, fw_hm_abs, fw_hm_err
@@ -229,11 +233,8 @@ def gauss_model_2save(x_min, x_max, data, num_bin, *args):
 class GaussianMixtureModel(GaussianMixtureBase):
     """The class for implementing what we colloquially call the 'GMM' algorithms.
 
-    Functionality includes:
-        AIC and BIC
-        Cartesian or Polar Coordinates
-        Spherical, Tied, Diagonal, and Full covariance matrices
-        General or strict fits
+    Functionality includes: AIC or BIC information criteria, Cartesian or Polar Coordinates,
+    Spherical, Tied, Diagonal, or Full covariance matrices, and general or strict fits
 
     version : 0.1
 
@@ -372,7 +373,7 @@ class GaussianMixtureModel(GaussianMixtureBase):
                              "got %s instead." % self.coordinates)
 
     def _GMM_fit(self, x, n_components):
-        """The function that fits a Gaussian Mixture Model to the data given by x.
+        """The method that fits a Gaussian Mixture Model to the data given by x.
 
         Parameters
         x : array-like, shape (n_samples, n_attributes)
@@ -397,7 +398,10 @@ class GaussianMixtureModel(GaussianMixtureBase):
 
     def _calc_secondary_centers_unc(self, c1s, c1s_err, c2s,
                                     c2s_err, data_frame_object):
-        """Calculate the coordinates of the cluster centers for the coordinate system that was not used for the fit.
+        """Calculate the coordinates of the cluster centers for the coordinate system that
+        was not used for the fit.
+
+        Standard errors are calculated with typical standard error propagation methods.
 
         Parameters
         ----------
@@ -407,7 +411,7 @@ class GaussianMixtureModel(GaussianMixtureBase):
             of the cluster centers.
 
         c1s_err : array-like, shape (n_components,)
-            The uncertainty in the c1s.
+            The standard error in the c1s.
 
         c2s : array-like, shape (n_components,)
             The y-coordinates of the cluster centers if clustering
@@ -415,7 +419,7 @@ class GaussianMixtureModel(GaussianMixtureBase):
             of the cluster centers.
 
         c2s_err : array-like, shape (n_components,)
-            The uncertainty in the c2s.
+            The standard error in the c2s.
 
         data_frame_object : object from the class DataFrame
             The object that contains the processed data and
@@ -567,6 +571,8 @@ class GaussianMixtureModel(GaussianMixtureBase):
         curve to each dimension of each cluster and uses the
         statistics from the fits to calculate the centers.
 
+        This method was written by Dwaipayan Ray and Adrian Valverde.
+
         Parameters
         ----------
         data_frame_object : object from the class DataFrame
@@ -574,6 +580,10 @@ class GaussianMixtureModel(GaussianMixtureBase):
             information that is used by the algorithms to do the
             fits.
         """
+        if not self.clustered_:
+            raise NotImplementedError("Must run a method to cluster the "
+                                      "data before recalculating centers.")
+
         if not data_frame_object.phase_shifted_:
             shift_phase_dimension(data_frame_object)
 
@@ -915,7 +925,8 @@ class GaussianMixtureModel(GaussianMixtureBase):
 
         Given a data frame object that has already been used
         to generate histograms for each dimension of data, this
-        function will graph a GMM fit over each dimension.
+        method will graph a GMM fit over each dimension. The returned
+        matplotlib.plyplot figure can be shown and saved separately.
 
         Parameters
         ----------
@@ -975,6 +986,9 @@ class GaussianMixtureModel(GaussianMixtureBase):
     def plot_pdf_surface(self, data_frame_object):
         """Plot the pdf of the Gaussian mixture on a surface.
 
+        The returned matplotlib.plyplot figure can be shown and saved
+        separately.
+
         Parameters
         ----------
         data_frame_object : object from the class DataFrame
@@ -992,8 +1006,8 @@ class GaussianMixtureModel(GaussianMixtureBase):
             which is done separately.
         """
         if not self.clustered_:
-            print("Must cluster data before visualizing it.")
-            exit()
+            raise NotImplementedError("Must run a method to cluster the "
+                                      "data before visualization.")
 
         fig = plt.figure()
         ax = fig.gca(projection='3d')
@@ -1085,7 +1099,12 @@ class GaussianMixtureModel(GaussianMixtureBase):
         return fig, save_string
 
     def show_results(self, data_frame_object):
-        """Display the clustering results.
+        """Return the clustering results.
+
+        The returned matplotlib.plyplot figure can be shown and saved
+        separately.
+
+        This method was written by Dwaipayan Ray and Adrian Valverde.
 
         Parameters
         ----------
@@ -1103,6 +1122,10 @@ class GaussianMixtureModel(GaussianMixtureBase):
             The recommended file name to use when saving the plot,
             which is done separately.
         """
+        if not self.clustered_:
+            raise NotImplementedError("Must run a method to cluster the "
+                                      "data before visualization.")
+
         data_array = data_frame_object.data_array_
         n_samples = len(data_array[:, 0])
         center_array = self.centers_array_
